@@ -337,6 +337,10 @@ class py2app(Command):
         #    self.includes.add('pkgutil')
         #    self.includes.add('imp')
         self.packages = set(fancy_split(self.packages))
+        for pkg in self.packages:
+            if "." in pkg:
+                raise DistutilsOptionError("Cannot include subpackages using the 'packages' option")
+
         self.excludes = set(fancy_split(self.excludes))
         self.excludes.add('readline')
         # included by apptemplate
@@ -344,6 +348,12 @@ class py2app(Command):
         if getattr(self.distribution, 'install_requires', None):
             self.includes.add('pkg_resources')
             self.eggs = pkg_resources.require(self.distribution.install_requires)
+
+        # Setuptools/distribute style namespace packages uses
+        # __import__('pkg_resources'), and that import isn't detected at the
+        # moment. Forcefully include pkg_resources.
+        self.includes.add('pkg_resources')
+
         dylib_excludes = fancy_split(self.dylib_excludes)
         self.dylib_excludes = []
         for fn in dylib_excludes:
@@ -807,8 +817,6 @@ class py2app(Command):
         self.mkpath(self.temp_dir)
 
         self.dist_dir = os.path.abspath(self.dist_dir)
-        if os.path.exists(self.dist_dir):
-            shutil.rmtree(self.dist_dir)
         self.mkpath(self.dist_dir)
 
         self.lib_dir = os.path.join(self.bdist_dir,
